@@ -32,19 +32,18 @@ BEGIN {
 
     @ISA = qw(Exporter);
 
-    @EXPORT = qw($HOME $apt_policy $cwd $username $verbose $nolog
-		 $source_dependencies $mailprog $dpkg $sudo $su
+    @EXPORT = qw($HOME %alternatives $apt_policy $check_watches $cwd
+		 $username $verbose $nolog $mailprog $dpkg $su
 		 $schroot $schroot_options $fakeroot $apt_get
 		 $apt_cache $dpkg_source $md5sum $avg_time_db
 		 $avg_space_db $build_env_cmnd $pgp_options $log_dir
-		 $mailto $mailfrom $purge_build_directory
-		 @toolchain_regex $stalled_pkg_timeout
-		 $srcdep_lock_dir $srcdep_lock_wait $chroot_only
-		 $chroot_mode @ignore_watches_no_build_deps $build_dir
-		 $sbuild_mode $debug $force_orig_source
+		 $mailto $mailfrom @no_auto_upgrade
+		 $purge_build_directory @toolchain_regex
+		 $stalled_pkg_timeout $srcdep_lock_dir
+		 $srcdep_lock_wait @ignore_watches_no_build_deps
+		 $build_dir $sbuild_mode $debug $force_orig_source
 		 %individual_stalled_pkg_timeout $path
-		 $maintainer_name $uploader_name $key_id);
-}
+		 $maintainer_name $uploader_name %watches $key_id); }
 
 # Originally from the main namespace.
 (our $HOME = $ENV{'HOME'})
@@ -55,10 +54,11 @@ our $verbose = 0;
 our $nolog = 0;
 
 # Defaults.
-our $source_dependencies = "/etc/source-dependencies";
+# TODO: Remove $source_dependencies after Lenny.
+our $source_dependencies;
 our $mailprog = "/usr/sbin/sendmail";
 our $dpkg = "/usr/bin/dpkg";
-our $sudo = "/usr/bin/sudo";
+our $sudo;
 our $su = "/bin/su";
 our $schroot = "/usr/bin/schroot";
 our $schroot_options = "-q";
@@ -67,24 +67,29 @@ our $apt_get = "/usr/bin/apt-get";
 our $apt_cache = "/usr/bin/apt-cache";
 our $dpkg_source = "/usr/bin/dpkg-source";
 our $md5sum = "/usr/bin/md5sum";
-our $avg_time_db;
-our $avg_space_db;
+our $avg_time_db = "/var/lib/sbuild/avg-build-times";
+our $avg_space_db = "/var/lib/sbuild/avg-build-space";
 our $build_env_cmnd = "";
 our $pgp_options = "-us -uc";
 our $log_dir = "$HOME/logs";
 our $mailto = "";
 our $mailfrom = "Source Builder <sbuild>";
 our $purge_build_directory = "successful";
-our @toolchain_regex = ( 'binutils$', 'gcc-[\d.]+$', 'g\+\+-[\d.]+$', 'libstdc\+\+', 'libc[\d.]+-dev$', 'linux-kernel-headers$' );
-our $stalled_pkg_timeout = 90; # minutes
+our @toolchain_regex = ( 'binutils$', 'gcc-[\d.]+$', 'g\+\+-[\d.]+$', 'libstdc\+\+', 'libc[\d.]+-dev$', 'linux-kernel-headers$', 'linux-libc-dev$', 'gnumach-dev$', 'hurd-dev$', 'kfreebsd-kernel-headers$');
+our $stalled_pkg_timeout = 150; # minutes
 our $srcdep_lock_dir = "/var/lib/sbuild/srcdep-lock";
 our $srcdep_lock_wait = 1; # minutes
-our $chroot_only = 1;
-our $chroot_mode = "split";
+# TODO: Remove $chroot_only after Lenny
+our $chroot_only;
+# TODO: Remove $chroot_mode after Lenny
+our $chroot_mode;
 our $apt_policy = 1;
+our $check_watches = 1;
 our @ignore_watches_no_build_deps = qw();
+our %watches;
+# TODO: Remove $build_dir after Lenny
 our $build_dir = undef;
-our $sbuild_mode = "buildd";
+our $sbuild_mode = "user";
 our $debug = 0;
 our $force_orig_source = 0;
 our %individual_stalled_pkg_timeout = ();
@@ -92,9 +97,38 @@ our $path = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X
 our $maintainer_name;
 our $uploader_name;
 our $key_id;
+our %alternatives = ("info-browser"		=> "info",
+		     "httpd"			=> "apache",
+		     "postscript-viewer"	=> "ghostview",
+		     "postscript-preview"	=> "psutils",
+		     "www-browser"		=> "lynx",
+		     "awk"			=> "gawk",
+		     "c-shell"			=> "tcsh",
+		     "wordlist"			=> "wenglish",
+		     "tclsh"			=> "tcl8.4",
+		     "wish"			=> "tk8.4",
+		     "c-compiler"		=> "gcc",
+		     "fortran77-compiler"	=> "g77",
+		     "java-compiler"		=> "jikes",
+		     "libc-dev"			=> "libc6-dev",
+		     "libgl-dev"		=> "xlibmesa-gl-dev",
+		     "libglu-dev"		=> "xlibmesa-glu-dev",
+		     "libncurses-dev"		=> "libncurses5-dev",
+		     "libz-dev"			=> "zlib1g-dev",
+		     "libg++-dev"		=> "libstdc++6-4.0-dev",
+		     "emacsen"			=> "emacs21",
+		     "mail-transport-agent"	=> "ssmtp",
+		     "mail-reader"		=> "mailx",
+		     "news-transport-system"	=> "inn",
+		     "news-reader"		=> "nn",
+		     "xserver"			=> "xvfb",
+		     "mysql-dev"		=> "libmysqlclient-dev",
+		     "giflib-dev"		=> "libungif4-dev",
+		     "freetype2-dev"		=> "libttf-dev");
+
+our @no_auto_upgrade = qw(dpkg apt bash libc6 libc6-dev dpkg-dev);
 
 # read conf files
-require "/usr/share/sbuild/sbuild.conf";
 require "/etc/sbuild/sbuild.conf" if -r "/etc/sbuild/sbuild.conf";
 require "$HOME/.sbuildrc" if -r "$HOME/.sbuildrc";
 
@@ -102,33 +136,8 @@ sub init {
 	# some checks
 	die "mailprog binary $Sbuild::Conf::mailprog does not exist or isn't executable\n"
 		if !-x $Sbuild::Conf::mailprog;
-	if ($Sbuild::Conf::chroot_mode eq "split") {
-		die "sudo binary $Sbuild::Conf::sudo does not exist or isn't executable\n"
-			if !-x $Sbuild::Conf::sudo;
-
-		local (%ENV) = %ENV; # make local environment
-		$ENV{'DEBIAN_FRONTEND'} = "noninteractive";
-		$ENV{'APT_CONFIG'} = "test_apt_config";
-		$ENV{'SHELL'} = "/bin/sh";
-
-		chomp( my $test_df = `$Sbuild::Conf::sudo sh -c 'echo \$DEBIAN_FRONTEND'` );
-		chomp( my $test_ac = `$Sbuild::Conf::sudo sh -c 'echo \$APT_CONFIG'` );
-		chomp( my $test_sh = `$Sbuild::Conf::sudo sh -c 'echo \$SHELL'` );
-
-		if ($test_df ne "noninteractive" ||
-		    $test_ac ne "test_apt_config" ||
-		    $test_sh ne "/bin/sh") {
-			print STDERR "$Sbuild::Conf::sudo is stripping APT_CONFIG, DEBIAN_FRONTEND and/or SHELL from the environment\n";
-			print STDERR "'Defaults:$username env_keep+=\"APT_CONFIG DEBIAN_FRONTEND SHELL\"' is not set in /etc/sudoers\n";
-			die "$Sbuild::Conf::sudo is incorrectly configured"
-
-		}
-	} elsif ($Sbuild::Conf::chroot_mode eq "schroot") {
-		die "sudo binary $Sbuild::Conf::schroot does not exist or isn't executable\n"
-			if !-x $Sbuild::Conf::schroot;
-	} else {
-		die "Invalid chroot mode: $Sbuild::Conf::chroot_mode\n";
-	}
+	die "schroot binary $Sbuild::Conf::schroot does not exist or isn't executable\n"
+	    if !-x $Sbuild::Conf::schroot;
 	die "apt-get binary $Sbuild::Conf::apt_get does not exist or isn't executable\n"
 		if !-x $Sbuild::Conf::apt_get;
 	die "apt-cache binary $Sbuild::Conf::apt_cache does not exist or isn't executable\n"
@@ -137,6 +146,7 @@ sub init {
 		if !-x $Sbuild::Conf::dpkg_source;
 	die "$Sbuild::Conf::srcdep_lock_dir is not a directory\n"
 		if ! -d $Sbuild::Conf::srcdep_lock_dir;
+
 	die "mailto not set\n" if !$Sbuild::Conf::mailto;
 
 	if (!defined($Sbuild::Conf::build_dir)) {
@@ -146,6 +156,25 @@ sub init {
 	    die "Build directory $Sbuild::Conf::build_dir does not exist";
 	}
 
+	# TODO: Remove chroot_mode, chroot_only, sudo and
+	# source_dependencies after Lenny.
+
+	if (defined($chroot_mode)) {
+	    die "chroot_mode is obsolete";
+	}
+
+	if (defined($chroot_only)) {
+	    die "chroot_only is obsolete";
+	}
+
+	if (defined($sudo)) {
+	    die "sudo is obsolete";
+	}
+
+	if (defined($source_dependencies)) {
+	    die "Source dependencies are obsolete";
+	}
+
 }
 
 1;
